@@ -1,7 +1,7 @@
 /*
   determine_file_actions() is called by client() to determine what to
   do with each pair of files that is unequal. It asks the user (unless
-  r2sync is running in in batch mode) and returns lists of files that
+  r2sync is running in batch mode) and returns lists of files that
   need to be updated on each of the servers and a list of files that
   need to be deleted.
 
@@ -40,6 +40,7 @@
 #include "print.e"		/* Defines print() */
 #include "c-profile.e"		/* Defines struct Profile */
 #include "c-askuser.e"		/* Defines ask() */
+#include "c-autocheck.e"	/* Defines same_file() */
 
 
 /* mode_to_symbolic -- turn mode bits into a string */
@@ -105,6 +106,22 @@ static void display_file_info(Profile profile, const char *path)
 }
 
 
+/* check_dups -- print error & return false if any files differ only in case */
+static bool check_dups(const int root, const fileinfo files[], const int nfiles)
+{
+  int i;
+
+  for (i = 1; i < nfiles; i++)
+    if (strcasecmp(files[i-1].path, files[i].path) == 0) {
+      warnx(_("Root %1$d has files that map to the same file on root %2$d:\n"
+	  "  %3$s\n  %4$s"), root, 3 - root, files[i-1].path, files[i].path);
+      return false;
+    }
+
+  return true;
+}
+
+
 /* determine_file_actions -- ask user what to do with each file pair */
 EXPORT bool determine_file_actions(Profile profile,
 				   fileinfo files1[], int nfiles1,
@@ -119,6 +136,8 @@ EXPORT bool determine_file_actions(Profile profile,
   fileinfo *h1, *h2;
   int action;
 
+  if (!profile->quiet) printf(_("Comparing the lists...\n"));
+
   /* Allocate an array to store the default action for each file. This
      must be on the heap and not on the stack, because it may be
      bigger than the stack. */
@@ -132,6 +151,11 @@ EXPORT bool determine_file_actions(Profile profile,
   qsort(files2, nfiles2, sizeof(*files2),
 	profile->nocase ? cmpcase_fileinfo : cmp_fileinfo);
 
+  /* If one of the servers's file system is case-insensitive, we must
+     fail if the other server has files that differ only in case. */
+  if (profile->nocase && !check_dups(1, files1, nfiles1)) return false;
+  if (profile->nocase && !check_dups(2, files2, nfiles2)) return false;
+
   /* Determine default actions for each pair */
   for (i = j = k = 0; i < nfiles1 || j < nfiles2; ) {
 
@@ -141,33 +165,46 @@ EXPORT bool determine_file_actions(Profile profile,
     else r = strcmp(files1[i].path, files2[j].path);
 
     if (!is_excluded(profile, r < 0 ? files1[i].path : files2[j].path)) {
-      if (r < 0 && files1[i].status == 'd') {
-	/* Deleted on 1, absent on 2, so skip */
-      } else if (r < 0) {	/* No such file on root2 */
-	questions[k].h1 = files1 + i;
-	questions[k].h2 = NULL;
-	questions[k++].default_action = '>';
-      } else if (r > 0 && files2[j].status == 'd') {
-	/* Deleted on 2, absent on 1, so skip */
-      } else if (r > 0) {	/* No such file on root1 */
-	questions[k].h1 = NULL;
+      /*
+        root1 \ root2 | none  =  n  d  u  m 
+	--------------+--------------------
+	 none (r > 0) |  n/a  ?  <  .  ?  ?
+		 =    |    ?  .  ?  <  <  <
+		 n    |    >  ?  ?  ?  ?  ?
+		 d    |    .  >  ?  .  ?  ?
+		 u    |    ?  >  ?  ?  ?  ?
+		 m    |    ?  >  ?  ?  ?  ?
+
+         ? = ask user; . = do nothing; > = copy to 2; < = copy to 1
+       */
+      if ((r > 0 && files2[j].status == 'd') ||
+	(r < 0 && files1[i].status == 'd') ||
+	(r == 0 && files1[i].status == 'd' && files2[j].status == 'd') ||
+	(r == 0 && files1[i].status == '=' && files2[j].status == '=')) {
+	/* Absent/deleted on both or unchanged on both: skip */
+      } else if ((r > 0 && files2[j].status == 'n') ||
+	(r == 0 && files1[i].status == '=' && strchr("dum",files2[j].status))) {
+	/* New or modified on 2: copy to 1 */
+	questions[k].h1 = r > 0 ? NULL : files1 + i;
 	questions[k].h2 = files2 + j;
 	questions[k++].default_action = '<';
-      } else if (files1[i].status == 'd' && files2[j].status == 'd') {
-	/* Deleted on both sides, skip */
-      } else if (files1[i].status == '=' && files2[j].status == '=') {
-	/* Unchanged on both sides, skip */
-      } else if (files1[i].status == '=') { /* Only root2 changed */
+      } else if ((r < 0 && files1[i].status == 'n') ||
+	(r == 0 && strchr("dum",files1[i].status) && files2[j].status == '=')) {
+	/* New or modified on 1: copy to 2 */
 	questions[k].h1 = files1 + i;
-	questions[k].h2 = files2 + j;
-	questions[k++].default_action = '<';
-      } else if (files2[j].status == '=') { /* Only root1 changed */
-	questions[k].h1 = files1 + i;
-	questions[k].h2 = files2 + j;
+	questions[k].h2 = r < 0 ? NULL : files2 + j;
 	questions[k++].default_action = '>';
-      } else { 			/* Both sides changed in some way */
-	questions[k].h1 = files1 + i;
-	questions[k].h2 = files2 + j;
+      } else if (r == 0 && profile->autocheck &&
+	same_file(profile, &files1[i], &files2[j])) {
+	/* Both sides changed, but ended up being the
+	   same. Arbitrarily send from 1 to 2. */
+	*send_to2 = realloc(*send_to2, (*nsend2 + 1) * sizeof(**send_to2));
+	if (!*send_to2) {warn(NULL); free(questions); return false;}
+	(*send_to2)[(*nsend2)++] = files1[i];
+      } else {
+	/* Modified on both sides or damaged store files: ask user */
+	questions[k].h1 = r > 0 ? NULL : files1 + i;
+	questions[k].h2 = r < 0 ? NULL : files2 + j;
 	questions[k++].default_action = profile->conflict_resolution;
       }
     }
@@ -176,9 +213,17 @@ EXPORT bool determine_file_actions(Profile profile,
     if (r <= 0) i++;
   }
 
-  if (!profile->quiet)
-    printf(ngettext("Found %d file to synchronize\n\n",
-		    "Found %d files to synchronize\n\n", k), k);
+  if (!profile->quiet) {
+    if (*nsend2) {
+      printf(ngettext("Found %d updated but identical file\n",
+	  "Found %d updated but identical files\n", *nsend2), *nsend2);
+      printf(ngettext("Found %d different file\n\n",
+	  "Found %d different files\n\n", k), k);
+    } else {
+      printf(ngettext("Found %d file to synchronize\n\n",
+	  "Found %d files to synchronize\n\n", k), k);
+    }
+  }
 
   /* Ask the user what to do with each file, unless we're in batch_mode */
   if (!profile->batch_mode) {
@@ -242,6 +287,11 @@ EXPORT bool determine_file_actions(Profile profile,
 	if (action == 'g') {
 	  profile->batch_mode = true; /* And do not increment i */
 
+	} else if (action == 'G') {   /* Skip the remaining questions */
+	  i = k;		      /* End the inner loop */
+	  with_conflict = -1;	      /* End the outer loop */
+	  profile->batch_mode = true; /* Do not ask before synchronizing */
+
 	} else if (action == 'p') {   /* Back to previous */
 
 	  while (1) {
