@@ -1,5 +1,5 @@
 /*
-  file_checksum_and_digest() is called by compute_delta().
+  file_checksum_and_digest() is called by compute_delta() and by sums().
   It computes the checksum and the digest for a whole file.
 */
 
