@@ -6,8 +6,8 @@ set -vx
 # synchronization.
 
 trap 'rm -f $TMP1 $TMP2' 0
-TMP1=`mktemp /tmp/test-XXXX` || exit 1
-TMP2=`mktemp /tmp/test-XXXX` || exit 1
+TMP1=`mktemp /tmp/test-XXXXXX` || exit 1
+TMP2=`mktemp /tmp/test-XXXXXX` || exit 1
 
 ok=true
 
